import pandas as pd

from pandas_datareader.base import _BaseReader


class RobinhoodQuoteReader(_BaseReader):
    """
    Read quotes from Robinhood

    Parameters
    ----------
    symbols : {str, List[str]}
        String symbol of like of symbols
    start : None
        Quotes are near real-time and so this value is ignored
    end : None
        Quotes are near real-time and so this value is ignored
    retry_count : int, default 3
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq : None
        Quotes are near real-time and so this value is ignored
    """
    _format = 'json'

    def __init__(self, symbols, start=None, end=None, retry_count=3, pause=.1,
                 timeout=30, session=None, freq=None):
        super(RobinhoodQuoteReader, self).__init__(symbols, start, end,
                                                   retry_count, pause,
                                                   timeout, session, freq)
        if isinstance(self.symbols, str):
            self.symbols = [self.symbols]
        self._max_symbols = 1630
        self._validate_symbols()
        self._json_results = []

    def _validate_symbols(self):
        if len(self.symbols) > self._max_symbols:
            raise ValueError('A maximum of {0} symbols are supported '
                             'in a single call.'.format(self._max_symbols))

    def _get_crumb(self, *args):
        pass

    @property
    def url(self):
        """API URL"""
        return 'https://api.robinhood.com/quotes/'

    @property
    def params(self):
        """Parameters to use in API calls"""
        symbols = ','.join(self.symbols)
        return {'symbols': symbols}

    def _process_json(self):
        res = pd.DataFrame(self._json_results)
        return res.set_index('symbol').T

    def _read_lines(self, out):
        if 'next' in out:
            self._json_results.extend(out['results'])
            return self._read_one_data(out['next'])
        self._json_results.extend(out['results'])
        return self._process_json()


class RobinhoodHistoricalReader(RobinhoodQuoteReader):
    """
    Read historical values from Robinhood

    Parameters
    ----------
    symbols : {str, List[str]}
        String symbol of like of symbols
    start : None
        Ignored.  See span and interval.
    end : None
        Ignored.  See span and interval.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq : None
        Quotes are near real-time and so this value is ignored
    interval : {'day' ,'week', '5minute', '10minute'}
        Interval between historical prices
    span : {'day', 'week', 'year', '5year'}
        Time span relative to now to retrieve.  The available spans are a
        function of interval. See notes

    Notes
    -----
    Only provides up to 1 year of daily data.

    The available spans are a function of interval.

      * day: year
      * week: 5year
      * 5minute: day, week
      * 10minute: day, week
    """
    _format = 'json'

    def __init__(self, symbols, start=None, end=None, retry_count=3, pause=.1,
                 timeout=30, session=None, freq=None, interval='day',
                 span='year'):
        super(RobinhoodHistoricalReader, self).__init__(symbols, start, end,
                                                        retry_count, pause,
                                                        timeout, session, freq)
        interval_span = {'day': ['year'],
                         'week': ['5year'],
                         '10minute': ['day', 'week'],
                         '5minute': ['day', 'week']}
        if interval not in interval_span:
            raise ValueError('Interval must be one of '
                             '{0}'.format(', '.join(interval_span.keys())))
        valid_spans = interval_span[interval]
        if span not in valid_spans:
            raise ValueError('For interval {0}, span must '
                             'be in: {1}'.format(interval, valid_spans))
        self.interval = interval
        self.span = span
        self._max_symbols = 75
        self._validate_symbols()
        self._json_results = []

    @property
    def url(self):
        """API URL"""
        return 'https://api.robinhood.com/quotes/historicals/'

    @property
    def params(self):
        """Parameters to use in API calls"""
        symbols = ','.join(self.symbols)
        pars = {'symbols': symbols,
                'interval': self.interval,
                'span': self.span}

        return pars

    def _process_json(self):
        df = []
        for sym in self._json_results:
            vals = pd.DataFrame(sym['historicals'])
            vals['begins_at'] = pd.to_datetime(vals['begins_at'])
            vals['symbol'] = sym['symbol']
            df.append(vals.set_index(['symbol', 'begins_at']))
        return pd.concat(df, 0)



# pylint: disable-msg=E1101,W0613,W0603

import os

import numpy as np
import pandas as pd
import pandas.util.testing as tm
import pytest

from pandas_datareader.io import read_jsdmx
from pandas_datareader.compat import PANDAS_0210


class TestJSDMX(object):

    def setup_method(self, method):
        self.dirpath = tm.get_data_path()

    @pytest.mark.skipif(not PANDAS_0210, reason='Broken on old pandas')
    def test_tourism(self):
        # OECD -> Industry and Services -> Inbound Tourism
        result = read_jsdmx(os.path.join(self.dirpath, 'jsdmx',
                                         'tourism.json'))
        assert isinstance(result, pd.DataFrame)
        jp = result['Japan']
        visitors = ['China', 'Hong Kong, China',
                    'Total international arrivals',
                    'Korea', 'Chinese Taipei', 'United States']

        exp_col = pd.Index(
            ['China', 'Hong Kong, China', 'Total international arrivals',
             'Korea', 'Chinese Taipei', 'United States'],
            name='Variable')
        exp_idx = pd.DatetimeIndex(['2008-01-01', '2009-01-01', '2010-01-01',
                                    '2011-01-01', '2012-01-01', '2013-01-01',
                                    '2014-01-01', '2015-01-01', '2016-01-01'],
                                   name='Year')
        values = [
            [1000000.0, 550000.0, 8351000.0, 2382000.0, 1390000.0, 768000.0],
            [1006000.0, 450000.0, 6790000.0, 1587000.0, 1024000.0, 700000.0],
            [1413000.0, 509000.0, 8611000.0, 2440000.0, 1268000.0, 727000.0],
            [1043000.0, 365000.0, 6219000.0, 1658000.0, 994000.0, 566000.0],
            [1430000.0, 482000.0, 8368000.0, 2044000.0, 1467000.0, 717000.0],
            [1314000.0, 746000.0, 10364000.0, 2456000.0, 2211000.0, 799000.0],
            [2409000.0, 926000.0, 13413000.0, 2755000.0, 2830000.0, 892000.0],
            [4993689.0, 1524292.0, 19737409.0, 4002095.0, 3677075.0,
             1033258.0],
            [6373564.0, 1839193.0, 24039700.0, 5090302.0, 4167512.0, 1242719.0]
        ]
        values = np.array(values, dtype='object')
        expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
        tm.assert_frame_equal(jp[visitors], expected)

    @pytest.mark.skipif(not PANDAS_0210, reason='Broken on old pandas')
    def test_land_use(self):
        # OECD -> Environment -> Resources Land Use
        result = read_jsdmx(os.path.join(self.dirpath, 'jsdmx',
                                         'land_use.json'))
        assert isinstance(result, pd.DataFrame)
        result = result.loc['2010':'2011']

        cols = ['Arable land and permanent crops',
                'Arable and cropland % land area',
                'Total area', 'Forest', 'Forest % land area',
                'Land area', 'Permanent meadows and pastures',
                'Meadows and pastures % land area', 'Other areas',
                'Other % land area']
        exp_col = pd.MultiIndex.from_product([
            ['Japan', 'United States'],
            cols], names=['Country', 'Variable'])
        exp_idx = pd.DatetimeIndex(['2010', '2011'], name='Year')
        values = [
            [53790.0, 14.753154141525, 377800.0, np.nan, np.nan, 364600.0,
             5000.0, 1.3713658804169, np.nan, np.nan,
             1897990.0, 20.722767650476, 9629090.0, np.nan, np.nan, 9158960.0,
             2416000.0, 26.378540795025, np.nan,
             np.nan],
            [53580.0, 14.691527282698, 377800.0, np.nan, np.nan, 364700.0,
             5000.0, 1.3709898546751, np.nan, np.nan,
             1897990.0, 20.722767650476, 9629090.0, np.nan, np.nan, 9158960.0,
             2416000.0, 26.378540795025, np.nan,
             np.nan]]
        values = np.array(values)
        expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
        tm.assert_frame_equal(result[exp_col], expected)



# flake8: noqa

import datetime as dt

from pandas import read_csv, compat
from pandas.compat import StringIO

from pandas_datareader.base import _DailyBaseReader


class MoexReader(_DailyBaseReader):
    """
    Returns DataFrame of historical stock prices from symbols from Moex

    Parameters
    ----------
    symbols : str, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    start : str, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : str, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used

    Notes
    -----
    To avoid being penalized by Moex servers, pauses between downloading
    'chunks' of symbols can be specified.
    """

    def __init__(self, *args, **kwargs):
        super(MoexReader, self).__init__(*args, **kwargs)
        self.start = self.start.date()
        self.end_dt = self.end
        self.end = self.end.date()
        if not isinstance(self.symbols, compat.string_types):
            raise ValueError("Support for multiple symbols is not yet implemented.")

    __url_metadata = "https://iss.moex.com/iss/securities/{symbol}.csv"
    __url_data = "https://iss.moex.com/iss/history/engines/{engine}/" \
                 "markets/{market}/securities/{symbol}.csv"

    @property
    def url(self):
        """API URL"""
        return self.__url_data.format(
            engine=self.__engine,
            market=self.__market,
            symbol=self.symbols
        )

    def _get_params(self, start):
        params = {
            'iss.only': 'history',
            'iss.dp': 'point',
            'iss.df': '%Y-%m-%d',
            'iss.tf': '%H:%M:%S',
            'iss.dft': '%Y-%m-%d %H:%M:%S',
            'iss.json': 'extended',
            'callback': 'JSON_CALLBACK',
            'from': start,
            'till': self.end_dt.strftime('%Y-%m-%d'),
            'limit': 100,
            'start': 1,
            'sort_order': 'TRADEDATE',
            'sort_order_desc': 'asc'
        }
        return params

    def _get_metadata(self):
        """ get a market and an engine for a given symbol """
        response = self._get_response(
            self.__url_metadata.format(symbol=self.symbols)
        )
        text = self._sanitize_response(response)
        if len(text) == 0:
            service = self.__class__.__name__
            raise IOError("{} request returned no data; check URL for invalid "
                          "inputs: {}".format(service, self.__url_metadata))
        if isinstance(text, compat.binary_type):
            text = text.decode('windows-1251')
        else:
            text = text

        header_str = 'secid;boardid;'
        get_data = False
        for s in text.splitlines():
            if s.startswith(header_str):
                get_data = True
                continue
            if get_data and s != '':
                fields = s.split(';')
                return fields[5], fields[7]
        service = self.__class__.__name__
        raise IOError("{} request returned no metadata: {}\n"
                      "Typo in security symbol `{}`?".format(
            service,
            self.__url_metadata.format(symbol=self.symbols),
            self.symbols
        )
        )

    def read(self):
        """Read data"""
        try:
            self.__market, self.__engine = self._get_metadata()

            out_list = []
            date_column = None
            while True:  # read in loop with small date intervals
                if len(out_list) > 0:
                    if date_column is None:
                        date_column = out_list[0].split(';').index('TRADEDATE')

                    # get the last downloaded date
                    start_str = out_list[-1].split(';', 4)[date_column]
                    start = dt.datetime.strptime(start_str, '%Y-%m-%d').date()
                else:
                    start_str = self.start.strftime('%Y-%m-%d')
                    start = self.start

                if start >= self.end or start >= dt.date.today():
                    break

                params = self._get_params(start_str)
                strings_out = self._read_url_as_String(self.url, params) \
                                  .splitlines()[2:]
                strings_out = list(filter(lambda x: x.strip(), strings_out))

                if len(out_list) == 0:
                    out_list = strings_out
                    if len(strings_out) < 101:
                        break
                else:
                    out_list += strings_out[1:]  # remove CSV head line
                    if len(strings_out) < 100:
                        break
            str_io = StringIO('\r\n'.join(out_list))
            df = self._read_lines(str_io)
            return df
        finally:
            self.close()

    def _read_url_as_String(self, url, params=None):
        """ Open url (and retry) """
        response = self._get_response(url, params=params)
        text = self._sanitize_response(response)
        if len(text) == 0:
            service = self.__class__.__name__
            raise IOError("{} request returned no data; check URL for invalid "
                          "inputs: {}".format(service, self.url))
        if isinstance(text, compat.binary_type):
            out = text.decode('windows-1251')
        else:
            out = text
        return out

    def _read_lines(self, input):
        """ return pandas DataFrame from input """
        rs = read_csv(input, index_col='TRADEDATE', parse_dates=True, sep=';',
                      na_values=('-', 'null'))
        # Get rid of unicode characters in index name.
        try:
            rs.index.name = rs.index.name.decode(
                'unicode_escape').encode('ascii', 'ignore')
        except AttributeError:
            # Python 3 string has no decode method.
            rs.index.name = rs.index.name.encode('ascii', 'ignore').decode()
        return rs



# flake8: noqa
import sys
from distutils.version import LooseVersion
from io import BytesIO

import pandas as pd
import pandas.compat as compat
import pandas.io.common as com

PY3 = sys.version_info >= (3, 0)

PANDAS_VERSION = LooseVersion(pd.__version__)

PANDAS_0190 = (PANDAS_VERSION >= LooseVersion('0.19.0'))
PANDAS_0200 = (PANDAS_VERSION >= LooseVersion('0.20.0'))
PANDAS_0210 = (PANDAS_VERSION >= LooseVersion('0.21.0'))
PANDAS_0230 = (PANDAS_VERSION >= LooseVersion('0.23.0'))

if PANDAS_0190:
    from pandas.api.types import is_number
else:
    from pandas.core.common import is_number

if PANDAS_0200:
    from pandas.util.testing import assert_raises_regex

    def get_filepath_or_buffer(filepath_or_buffer, encoding=None,
                               compression=None):

        # Dictionaries are no longer considered valid inputs
        # for "get_filepath_or_buffer" starting in pandas >= 0.20.0
        if isinstance(filepath_or_buffer, dict):
            return filepath_or_buffer, encoding, compression

        return com.get_filepath_or_buffer(
            filepath_or_buffer, encoding=encoding, compression=None)
else:
    from pandas.util.testing import assertRaisesRegexp as assert_raises_regex
    get_filepath_or_buffer = com.get_filepath_or_buffer

if PANDAS_0230:
    from pandas.core.dtypes.common import is_list_like
else:
    from pandas.core.common import is_list_like


if compat.PY3:
    from urllib.error import HTTPError
else:
    from urllib2 import HTTPError



import numpy as np
import pandas as pd
import pytest

from pandas_datareader.robinhood import RobinhoodQuoteReader, \
    RobinhoodHistoricalReader

syms = ['GOOG', ['GOOG', 'AAPL']]
ids = list(map(str, syms))


@pytest.fixture(params=['GOOG', ['GOOG', 'AAPL']], ids=ids)
def symbols(request):
    return request.param


def test_robinhood_quote(symbols):
    df = RobinhoodQuoteReader(symbols=symbols).read()
    assert isinstance(df, pd.DataFrame)
    if isinstance(symbols, str):
        symbols = [symbols]
    assert df.shape[1] == len(symbols)


def test_robinhood_quote_too_many():
    syms = np.random.randint(65, 90, size=(10000, 4)).tolist()
    syms = list(map(lambda r: ''.join(map(chr, r)), syms))
    syms = list(set(syms))
    with pytest.raises(ValueError):
        RobinhoodQuoteReader(symbols=syms)


def test_robinhood_historical_too_many():
    syms = np.random.randint(65, 90, size=(10000, 4)).tolist()
    syms = list(map(lambda r: ''.join(map(chr, r)), syms))
    syms = list(set(syms))
    with pytest.raises(ValueError):
        RobinhoodHistoricalReader(symbols=syms)
    with pytest.raises(ValueError):
        RobinhoodHistoricalReader(symbols=syms[:76])


def test_robinhood_historical(symbols):
    df = RobinhoodHistoricalReader(symbols=symbols).read()
    assert isinstance(df, pd.DataFrame)
    if isinstance(symbols, str):
        symbols = [symbols]
    assert df.index.levels[0].shape[0] == len(symbols)



from pandas_datareader.compat import is_list_like

from pandas import concat, read_csv

from pandas_datareader.base import _BaseReader


class FredReader(_BaseReader):
    """
    Get data for the given name from the St. Louis FED (FRED).
    """

    @property
    def url(self):
        """API URL"""
        return "https://fred.stlouisfed.org/graph/fredgraph.csv"

    def read(self):
        """Read data

        Returns
        -------
        data : DataFrame
            If multiple names are passed for "series" then the index of the
            DataFrame is the outer join of the indicies of each series.
        """
        try:
            return self._read()
        finally:
            self.close()

    def _read(self):
        if not is_list_like(self.symbols):
            names = [self.symbols]
        else:
            names = self.symbols

        urls = ["{}?id={}".format(self.url, n) for n in names]

        def fetch_data(url, name):
            """Utillity to fetch data"""
            resp = self._read_url_as_StringIO(url)
            data = read_csv(resp, index_col=0, parse_dates=True,
                            header=None, skiprows=1, names=["DATE", name],
                            na_values='.')
            try:
                return data.truncate(self.start, self.end)
            except KeyError:  # pragma: no cover
                if data.ix[3].name[7:12] == 'Error':
                    raise IOError("Failed to get the data. Check that "
                                  "{0!r} is a valid FRED series.".format(name))
                raise
        df = concat([fetch_data(url, n) for url, n in zip(urls, names)],
                    axis=1, join='outer')
        return df



import numpy as np
import pandas as pd
import pandas.util.testing as tm
import pandas_datareader.data as web

from pandas_datareader.compat import assert_raises_regex


class TestEurostat(object):

    def test_get_cdh_e_fos(self):
        # Employed doctorate holders in non managerial and non professional
        # occupations by fields of science (%)
        df = web.DataReader('cdh_e_fos', 'eurostat',
                            start=pd.Timestamp('2005-01-01'),
                            end=pd.Timestamp('2010-01-01'))

        assert isinstance(df, pd.DataFrame)
        assert df.shape == (2, 336)

        df = df['Percentage']['Total']['Natural sciences']
        df = df[['Norway', 'Poland', 'Portugal', 'Russia']]

        exp_col = pd.MultiIndex.from_product([['Norway', 'Poland', 'Portugal',
                                             'Russia'], ['Annual']],
                                             names=['GEO', 'FREQ'])
        exp_idx = pd.DatetimeIndex(['2006-01-01', '2009-01-01'],
                                   name='TIME_PERIOD')

        values = np.array([[25.49, np.nan, 39.05, np.nan],
                           [20.38, 25.1, 27.77, 38.1]])
        expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
        tm.assert_frame_equal(df, expected)

    def test_get_sts_cobp_a(self):
        # Building permits - annual data (2010 = 100)
        df = web.DataReader('sts_cobp_a', 'eurostat',
                            start=pd.Timestamp('2000-01-01'),
                            end=pd.Timestamp('2013-01-01'))

        idx = pd.date_range('2000-01-01', '2013-01-01', freq='AS',
                            name='TIME_PERIOD')
        ne_name = ('Index, 2010=100',
                   'Building permits - m2 of useful floor area',
                   'Unadjusted data (i.e. neither seasonally adjusted nor '
                   'calendar adjusted data)',
                   'Non-residential buildings, except office buildings',
                   'Netherlands', 'Annual')
        ne_values = [200.0, 186.5, 127.3, 130.7, 143.3, 147.8, 176.7,
                     227.4, 199.4, 128.5, 100.0, 113.8, 89.3, 77.6]
        ne = pd.Series(ne_values, name=ne_name, index=idx)

        uk_name = ('Index, 2010=100',
                   'Building permits - m2 of useful floor area',
                   'Unadjusted data (i.e. neither seasonally adjusted nor '
                   'calendar adjusted data)',
                   'Non-residential buildings, except office buildings',
                   'United Kingdom', 'Annual')
        uk_values = [112.5, 113.3, 110.2, 112.1, 119.1, 112.7, 113.1,
                     121.8, 114.0, 105.9, 100.0, 98.6, 103.7, 81.3]
        uk = pd.Series(uk_values, name=uk_name, index=idx)

        for expected in [ne, uk]:
            result = df[expected.name]
            tm.assert_series_equal(result, expected)

    def test_get_nrg_pc_202(self):
        # see gh-149

        df = web.DataReader('nrg_pc_202', 'eurostat',
                            start=pd.Timestamp('2010-01-01'),
                            end=pd.Timestamp('2013-01-01'))

        name = ('All taxes and levies included',
                'Gigajoules (gross calorific value = GCV)',
                'Euro',
                'Band D1 : Consumption < 20 GJ',
                'Natural gas', 'Denmark', 'Semi-annual')

        exp_index = pd.Index(['2013-B2', '2013-B1', '2012-B2', '2012-B1',
                              '2011-B2', '2011-B1', '2010-B2', '2010-B1'],
                             name='TIME_PERIOD')
        exp = pd.Series([27.1403, 27.5854, 26.5285, 27.2187,
                         28.5862, 28.6448, 26.8147, 26.4979],
                        index=exp_index, name=name)

        tm.assert_series_equal(df[name], exp)

    def test_get_prc_hicp_manr_exceeds_limit(self):
        # see gh-149
        msg = 'Query size exceeds maximum limit'
        with assert_raises_regex(ValueError, msg):
            web.DataReader('prc_hicp_manr', 'eurostat',
                           start=pd.Timestamp('2000-01-01'),
                           end=pd.Timestamp('2013-01-01'))



import os

import pandas as pd

from pandas_datareader.base import _BaseReader


def get_tiingo_symbols():
    """
    Get the set of stock symbols supported by Tiingo

    Returns
    -------
    symbols : DataFrame
        DataFrame with symbols (ticker), exchange, asset type, currency and
        start and end dates

    Notes
    -----
    Reads https://apimedia.tiingo.com/docs/tiingo/daily/supported_tickers.zip
    """
    url = 'https://apimedia.tiingo.com/docs/tiingo/daily/supported_tickers.zip'
    return pd.read_csv(url)


class TiingoDailyReader(_BaseReader):
    """
    Historical daily data from Tiingo on equities, ETFs and mutual funds

    Parameters
    ----------
    symbols : {str, List[str]}
        String symbol of like of symbols
    start : str, (defaults to '1/1/2010')
        Starting date, timestamp. Parses many different kind of date
        representations (e.g., 'JAN-01-2010', '1/1/10', 'Jan, 1, 1980')
    end : str, (defaults to today)
        Ending date, timestamp. Same format as starting date.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq : {str, None}
        Not used.
    api_key : str, optional
        Tiingo API key . If not provided the environmental variable
        TIINGO_API_KEY is read. The API key is *required*.
    """

    def __init__(self, symbols, start=None, end=None, retry_count=3, pause=0.1,
                 timeout=30, session=None, freq=None, api_key=None):
        super(TiingoDailyReader, self).__init__(symbols, start, end,
                                                retry_count, pause, timeout,
                                                session, freq)
        if isinstance(self.symbols, str):
            self.symbols = [self.symbols]
        self._symbol = ''
        if api_key is None:
            api_key = os.getenv('TIINGO_API_KEY')
        if not api_key or not isinstance(api_key, str):
            raise ValueError('The tiingo API key must be provided either '
                             'through the api_key variable or through the '
                             'environmental variable TIINGO_API_KEY.')
        self.api_key = api_key
        self._concat_axis = 0

    @property
    def url(self):
        """API URL"""
        _url = 'https://api.tiingo.com/tiingo/daily/{ticker}/prices'
        return _url.format(ticker=self._symbol)

    @property
    def params(self):
        """Parameters to use in API calls"""
        return {'startDate': self.start.strftime('%Y-%m-%d'),
                'endDate': self.end.strftime('%Y-%m-%d'),
                'format': 'json'}

    def _get_crumb(self, *args):
        pass

    def _read_one_data(self, url, params):
        """ read one data from specified URL """
        headers = {'Content-Type': 'application/json',
                   'Authorization': 'Token ' + self.api_key}
        out = self._get_response(url, params=params, headers=headers).json()
        return self._read_lines(out)

    def _read_lines(self, out):
        df = pd.DataFrame(out)
        df['symbol'] = self._symbol
        df['date'] = pd.to_datetime(df['date'])
        df = df.set_index(['symbol', 'date'])
        return df

    def read(self):
        """Read data from connector"""
        dfs = []
        for symbol in self.symbols:
            self._symbol = symbol
            try:
                dfs.append(self._read_one_data(self.url, self.params))
            finally:
                self.close()
        return pd.concat(dfs, self._concat_axis)


class TiingoMetaDataReader(TiingoDailyReader):
    """
    Read metadata about symbols from Tiingo

    Parameters
    ----------
    symbols : {str, List[str]}
        String symbol of like of symbols
    start : str, (defaults to '1/1/2010')
        Not used.
    end : str, (defaults to today)
        Not used.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq : {str, None}
        Not used.
    api_key : str, optional
        Tiingo API key . If not provided the environmental variable
        TIINGO_API_KEY is read. The API key is *required*.
    """

    def __init__(self, symbols, start=None, end=None, retry_count=3, pause=0.1,
                 timeout=30, session=None, freq=None, api_key=None):
        super(TiingoMetaDataReader, self).__init__(symbols, start, end,
                                                   retry_count, pause, timeout,
                                                   session, freq, api_key)
        self._concat_axis = 1

    @property
    def url(self):
        """API URL"""
        _url = 'https://api.tiingo.com/tiingo/daily/{ticker}'
        return _url.format(ticker=self._symbol)

    @property
    def params(self):
        return None

    def _read_lines(self, out):
        s = pd.Series(out)
        s.name = self._symbol
        return s


class TiingoQuoteReader(TiingoDailyReader):
    """
    Read quotes (latest prices) from Tiingo

    Parameters
    ----------
    symbols : {str, List[str]}
        String symbol of like of symbols
    start : str, (defaults to '1/1/2010')
        Not used.
    end : str, (defaults to today)
        Not used.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : float, default 0.1
        Time, in seconds, of the pause between retries.
    session : Session, default None
        requests.sessions.Session instance to be used
    freq : {str, None}
        Not used.
    api_key : str, optional
        Tiingo API key . If not provided the environmental variable
        TIINGO_API_KEY is read. The API key is *required*.

    Notes
    -----
    This is a special case of the daily reader which automatically selected
    the latest data available for each symbol.
    """

    @property
    def params(self):
        return None



import pandas as pd
import pandas.util.testing as tm
import pytest

import pandas_datareader.data as web
from pandas_datareader.exceptions import ImmediateDeprecationError


class TestEdgarIndex(object):

    @classmethod
    def setup_class(cls):
        # As of December 31, SEC has disabled ftp access to EDGAR.
        # Disabling tests until re-write.
        pytest.skip("Disabling tests until re-write.")

    def test_raises(self):
        with pytest.raises(ImmediateDeprecationError):
            web.DataReader('full', 'edgar-index')

    def test_get_full_index(self):
        ed = web.DataReader('full', 'edgar-index')
        assert len(ed) > 1000

        exp_columns = pd.Index(['cik', 'company_name', 'form_type',
                                'date_filed', 'filename'], dtype='object')
        tm.assert_index_equal(ed.columns, exp_columns)

    def test_get_nonzip_index_and_low_date(self):
        ed = web.DataReader('daily', 'edgar-index', '1994-06-30',
                            '1994-07-02')
        assert len(ed) > 200
        assert ed.index.nlevels == 2

        dti = ed.index.get_level_values(0)
        assert isinstance(dti, pd.DatetimeIndex)
        exp_columns = pd.Index(['company_name', 'form_type',
                                'filename'], dtype='object')
        tm.assert_index_equal(ed.columns, exp_columns)

    def test_get_gz_index_and_no_date(self):
        # TODO: Rewrite, as this test causes Travis to timeout.

        ed = web.DataReader('daily', 'edgar-index')
        assert len(ed) > 2000

    def test_6_digit_date(self):
        ed = web.DataReader('daily', 'edgar-index', start='1998-05-18',
                            end='1998-05-18')
        assert len(ed) < 1200
        assert ed.index.nlevels == 2

        dti = ed.index.get_level_values(0)
        assert isinstance(dti, pd.DatetimeIndex)
        assert dti[0] == pd.Timestamp('1998-05-18')
        assert dti[-1] == pd.Timestamp('1998-05-18')

        exp_columns = pd.Index(['company_name', 'form_type',
                                'filename'], dtype='object')
        tm.assert_index_equal(ed.columns, exp_columns)



from pandas_datareader.base import _DailyBaseReader


class StooqDailyReader(_DailyBaseReader):

    """
    Returns DataFrame/Panel of historical stock prices from symbols, over date
    range, start to end. To avoid being penalized by Google Finance servers,
    pauses between downloading 'chunks' of symbols can be specified.

    Parameters
    ----------
    symbols : string, array-like object (list, tuple, Series), or DataFrame
        Single stock symbol (ticker), array-like object of symbols or
        DataFrame with index containing stock symbols.
    retry_count : int, default 3
        Number of times to retry query request.
    pause : int, default 0
        Time, in seconds, to pause between consecutive queries of chunks. If
        single value given for symbol, represents the pause between retries.
    chunksize : int, default 25
        Number of symbols to download consecutively before intiating pause.
    session : Session, default None
        requests.sessions.Session instance to be used

    Notes
    -----
    See `Stooq <https://stooq.com>`__
    """

    @property
    def url(self):
        """API URL"""
        return 'https://stooq.com/q/d/l/'

    def _get_params(self, symbol):
        params = {
            's': symbol,
            'i': "d"
        }
        return params
